// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: IDayCounter.cs,v $
 *  Revision 1.1.1.1  2003/02/01 18:25:25  jens
 *  Initial import
 *
 *  Revision 1.4  2002/08/30 18:21:10  stochastix
 *  license and namespace tweaks
 *
 *  Revision 1.3  2002/07/24 17:04:24  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.2  2002/07/24 14:42:42  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using Stochastix.Utilities;

namespace Stochastix.Business {

	/// <summary>
	/// This interface provides methods for determining the length of a time
	/// period according to given market convention, both as a number
	/// of days and as a year fraction.
	/// </summary>
	/// <remarks>
	/// This interface is mainly for COM interop.
	/// </remarks>
	[ RcsId("$Id: IDayCounter.cs,v 1.1.1.1 2003/02/01 18:25:25 jens Exp $") ] 
	[ ComVisible(true) ]
	[ InterfaceTypeAttribute(ComInterfaceType.InterfaceIsDual) ]
	[
		Description("This interface provides methods for determining the " +
			"length of a time period according to given market convention, " +
			"both as a number of days and as a year fraction.")
	]
	public interface IDayCounter  {
		
		/// <summary>
		/// A stringified representation of this DayCounter.
		/// </summary>
		/// <remarks>
		/// This method is used for output and comparison between indexes.
		///	It is <b>not</b> meant to be used for writing switch-on-type code.
		/// </remarks>
		/// <returns>A String representing the object.</returns>
		[
		Description("A stringified representation of this object."),
		DispId(0)	// DISPID_VALUE
		]
		String ToString();

		/// <summary>
		/// The number of days between two dates.
		/// </summary>
		/// <param name="startDate">The start date.</param>
		/// <param name="endDate">The end date.</param>
		/// <returns>The number of days</returns>
		[
			Description("The number of days between two dates.")
		]
		int DayCount(DateTime startDate, DateTime endDate);

		/// <summary>
		/// The period between two dates as a fraction of year.
		/// </summary>
		/// <param name="startDate">The start date.</param>
		/// <param name="endDate">The end date.</param>
		/// <returns>The period between two dates as a fraction of year.</returns>
		[
			ComVisible(false),
			Description("The period between two dates as a fraction of year.")
		]
		double YearFraction(DateTime startDate, DateTime endDate);

		/// <summary>
		/// Returns the period between two dates as a fraction of year.
		/// </summary>
		/// <param name="startDate">The start date.</param>
		/// <param name="endDate">The end date.</param>
		/// <param name="refPeriodStart">Begin of the reference period or 0.</param>
		/// <param name="refPeriodEnd">End of the erefernce period or 0.</param>
		/// <returns>The period between two dates as a fraction of year.</returns>
		[
			Description("The period between two dates as a fraction of year.")
		]
		double YearFraction(DateTime startDate, DateTime endDate, 
			DateTime refPeriodStart, DateTime refPeriodEnd);
	}

}

