// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: TestDayCounters.cs,v $
 *  Revision 1.1.1.1  2003/02/01 18:25:26  jens
 *  Initial import
 *
 *
 *********************************************************************
 */
#endregion

namespace Stochastix.Business.Tests
{
	using System;
	using System.Collections;
	using NUnit.Framework;
	using Stochastix.Business;
	using Stochastix.Business.Calendars;
	using Stochastix.Business.DayCounters;

	/// <summary>
	/// 
	/// </summary>
	public class TestDayCounters: TestCase 
	{
		/// <summary>
		/// Just a constructor. I think you can pass the name of the
		/// test (==method) to run.
		/// </summary>
		/// <param name="name">Name of the test to run.</param>
		public TestDayCounters(String name): base(name) {}

		// declare private members here used for test setup

		/// <summary>
		/// Initialize this test case.
		/// </summary>
		protected override void SetUp() 
		{
		}

		/// <summary>
		/// Free all you resources.
		/// </summary>
		protected override void TearDown()
		{
		}

		/// <summary>
		/// Testing ActualActual day counters with reference examples from
		/// <see href="http://www.isda.org/c_and_a/pdf/mktc1198.pdf">ISDA</see>.
		/// </summary>
		public void TestActualActual()
		{

			ArrayList cases = new ArrayList(20);

#region actual/actual test data from mktc1198.pdf

			// First example

			cases.Add( new Object[] 
				{
					"First example", ActualActualISDA.Instance, 
					0.497724380567, null,
					new DateTime(2003, 11, 1), new DateTime( 2004, 5, 1),
					CalendarBase.NullDate, CalendarBase.NullDate
				});
			cases.Add( new Object[] 
				{
					"First example", ActualActualISMA.Instance, 
					0.500000000000, null,
					new DateTime(2003, 11, 1), new DateTime( 2004, 5, 1),
					new DateTime(2003, 11, 1), new DateTime( 2004, 5, 1)
				});
			cases.Add( new Object[] 
				{
					"First example", ActualActualAFB.Instance, 
					0.497267759563, null,
					new DateTime(2003, 11, 1), new DateTime( 2004, 5, 1),
					CalendarBase.NullDate, CalendarBase.NullDate
				});

			// Short first calculation period (first period)

			cases.Add( new Object[] 
				{
					"Short first calculation period (first period)", 
					ActualActualISDA.Instance, 0.410958904110, null,
					new DateTime(1999, 2, 1), new DateTime( 1999, 7, 1),
					CalendarBase.NullDate, CalendarBase.NullDate
				});
			cases.Add( new Object[] 
				{
					"Short first calculation period (first period)", 
					ActualActualISMA.Instance, 0.410958904110, null,
					new DateTime(1999, 2, 1), new DateTime( 1999, 7, 1),
					new DateTime(1998, 7, 1), new DateTime( 1999, 7, 1),
				});
			cases.Add( new Object[] 
				{
					"Short first calculation period (first period)", 
					ActualActualAFB.Instance, 0.410958904110, null,
					new DateTime(1999, 2, 1), new DateTime( 1999, 7, 1),
					CalendarBase.NullDate, CalendarBase.NullDate
				});

			// Short first calculation period (second period)

			cases.Add( new Object[] 
				{
					"Short first calculation period (second period)", 
					ActualActualISDA.Instance, 1.001377348600, null,
					new DateTime(1999, 7, 1), new DateTime( 2000, 7, 1),
					CalendarBase.NullDate, CalendarBase.NullDate
				});

			cases.Add( new Object[] 
				{
					"Short first calculation period (second period)", 
					ActualActualISMA.Instance, 1.000000000000, null,
					new DateTime(1999, 7, 1), new DateTime( 2000, 7, 1),
					new DateTime(1999, 7, 1), new DateTime( 2000, 7, 1)
				});
			cases.Add( new Object[] 
				{
					"Short first calculation period (second period)", 
					ActualActualAFB.Instance, 1.000000000000, null,
					new DateTime(1999, 7, 1), new DateTime( 2000, 7, 1),
					CalendarBase.NullDate, CalendarBase.NullDate
				});

			// Long first calculation period (first period)

			cases.Add( new Object[] 
				{
					"Long first calculation period (first period)", 
					ActualActualISDA.Instance, 0.915068493151, null,
					new DateTime(2002, 8, 15), new DateTime( 2003, 7, 15),
					CalendarBase.NullDate, CalendarBase.NullDate
				});
			cases.Add( new Object[] 
				{
					"Long first calculation period (first period)", 
					ActualActualISMA.Instance, 0.915760869565, null,
					new DateTime(2002, 8, 15), new DateTime( 2003, 7, 15),
					new DateTime(2003, 1, 15), new DateTime( 2003, 7, 15)
				});
			cases.Add( new Object[] 
				{
					"Long first calculation period (first period)", 
					ActualActualAFB.Instance, 0.915068493151, null,
					new DateTime(2002, 8, 15), new DateTime( 2003, 7, 15),
					CalendarBase.NullDate, CalendarBase.NullDate
				});
			
			// Long first calculation period (second period)

			cases.Add( new Object[] 
				{	// case is in disagreement with mktc1198.pdf !!!!
					"Long first calculation period (second period)", 
					ActualActualISDA.Instance, 0.504004790778, null,
					new DateTime(2003, 7, 15), new DateTime( 2004, 1, 15),
					CalendarBase.NullDate, CalendarBase.NullDate
				});
			cases.Add( new Object[] 
				{
					"Long first calculation period (second period)", 
					ActualActualISMA.Instance, 0.500000000000, null,
					new DateTime(2003, 7, 15), new DateTime( 2004, 1, 15),
					new DateTime(2003, 7, 15), new DateTime( 2004, 1, 15)
				});
			cases.Add( new Object[] 
				{
					"Long first calculation period (second period)", 
					ActualActualAFB.Instance, 0.504109589041, null,
					new DateTime(2003, 7, 15), new DateTime( 2004, 1, 15),
					CalendarBase.NullDate, CalendarBase.NullDate
				});

			// Short final calculation period (penultimate period)

			cases.Add( new Object[] 
				{
					"Short final calculation period (penultimate period)", 
					ActualActualISDA.Instance, 0.503892506924, null,
					new DateTime(1999, 7, 30), new DateTime( 2000, 1, 30),
					CalendarBase.NullDate, CalendarBase.NullDate
				});
			cases.Add( new Object[] 
				{
					"Short final calculation period (penultimate period)", 
					ActualActualISMA.Instance, 0.500000000000, null,
					new DateTime(1999, 7, 30), new DateTime( 2000, 1, 30),
					new DateTime(1999, 7, 30), new DateTime( 2000, 1, 30)
				});
			cases.Add( new Object[] 
				{
					"Short final calculation period (penultimate period)", 
					ActualActualAFB.Instance, 0.504109589041, null,
					new DateTime(1999, 7, 30), new DateTime( 2000, 1, 30),
					CalendarBase.NullDate, CalendarBase.NullDate
				});

			// Short final calculation period (final period)

			cases.Add( new Object[] 
				{
					"Short final calculation period (final period)", 
					ActualActualISDA.Instance, 0.415300546448, null,
					new DateTime(2000, 1, 30), new DateTime( 2000, 6, 30),
					CalendarBase.NullDate, CalendarBase.NullDate
				});
 			cases.Add( new Object[] 
				{
					"Short final calculation period (final period)", 
					ActualActualISMA.Instance, 0.417582417582, null,
					new DateTime(2000, 1, 30), new DateTime( 2000, 6, 30),
					new DateTime(2000, 1, 30), new DateTime( 2000, 7, 30)
				});
			cases.Add( new Object[] 
				{
					"Short final calculation period (final period)", 
					ActualActualAFB.Instance, 0.415300546448, null,
					new DateTime(2000, 1, 30), new DateTime( 2000, 6, 30),
					CalendarBase.NullDate, CalendarBase.NullDate
				});

#endregion

			// check all test cases

			foreach (Object[] item in cases) 
			{
				IDayCounter dc = (IDayCounter)item[1];
				DateTime startDate = (DateTime)item[4];
				DateTime endDate = (DateTime)item[5];

				string testName = string.Format("{0}, day counter '{1}'",
					item[0], item[1]);

				double t = dc.YearFraction(startDate, endDate, 
					(DateTime)item[6], (DateTime)item[7]);
				Assertion.Assert(testName, item[2]==null ||	
					Math.Abs(t - (double)item[2])<1e-10
				);

				int days = dc.DayCount(startDate, endDate);
				Assertion.Assert(testName, item[3]==null ||	
					(int)item[3] == days
				);
			}

		}

	}

}
