// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: GammaDistribution.cs,v $
 *  Revision 1.1.1.1  2003/02/01 18:42:10  jens
 *  Initial import
 *
 *  Revision 1.4  2002/12/05 15:57:52  stochastix
 *  synced with my current sandbox
 *
 *  Revision 1.3  2002/10/15 12:43:00  stochastix
 *  updated license
 *
 *  Revision 1.2  2002/08/28 15:35:51  stochastix
 *  changes due to FxCop
 *
 *  Revision 1.1.1.1  2002/08/15 11:44:10  stochastix
 *  Initial revision
 *
 *  Revision 1.1  2002/07/26 00:25:00  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using Stochastix.Utilities;
using Stochastix.Numerics.Resources;

namespace Stochastix.Numerics.Distributions
{
	/// <summary>
	/// Gamma distribution.
	/// </summary>
	[ RcsId("$Id: GammaDistribution.cs,v 1.1.1.1 2003/02/01 18:42:10 jens Exp $") ] 
	[ ComVisible(false) ]
	public class GammaDistribution : IUnaryFunction
	{
		public GammaDistribution(double a) 
		{
			if( a <= 0.0 ) 
				// Invalid parameter a={0} for gamma distribution, must be a>0.0
				throw new ArgumentOutOfRangeException( "a", a,
					Strings.Format(GetType(), "DistAGr0", a) );
			this.a = a;					// 
		}
		
		private double a;

		public double Value(double x)
		{
		
			if (x <= 0.0) return 0.0;

            double gln = GammaFunction.LogValue(a);

            if ( x < (a+1.0) ) 
			{
                double ap = a;
                double del = 1.0/a;
                double sum = del;
                for (int n=1; n<=100; n++) 
				{
                    ap  += 1.0;
                    del *= x/ap;
                    sum += del;
                    if ( Math.Abs(del) < Math.Abs(sum)*3.0e-7 )
                        return sum*Math.Exp( -x + a*Math.Log(x) - gln );
                }
            } else {
                double b = x + 1.0 - a;
                double c = Double.MaxValue;
                double d = 1.0/b;
                double h = d;
                for (int n=1; n<=100; n++) 
				{
                    double an = -1.0*n*(n-a);
                    b += 2.0;
                    d = an*d + b;
                    if (Math.Abs(d) < Double.Epsilon) d = Double.Epsilon;
                    c = b + an/c;
                    if (Math.Abs(c) < Double.Epsilon) c = Double.Epsilon;
                    d = 1.0/d;
                    double del = d*c;
                    h *= del;
                    if (Math.Abs(del - 1.0) < Double.Epsilon) 
                        return h*Math.Exp(-x + a*Math.Log(x) - gln);
                }
            }
			// Too much iterations.
            throw new ApplicationException( Strings.Get( GetType(), "DistIter"));	
        }
	}
}
