// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: NonCentralChiSquareDistribution.cs,v $
 *  Revision 1.1.1.1  2003/02/01 18:42:10  jens
 *  Initial import
 *
 *  Revision 1.4  2002/12/05 15:57:52  stochastix
 *  synced with my current sandbox
 *
 *  Revision 1.3  2002/10/15 12:43:00  stochastix
 *  updated license
 *
 *  Revision 1.2  2002/08/28 15:35:51  stochastix
 *  changes due to FxCop
 *
 *  Revision 1.1.1.1  2002/08/15 11:44:10  stochastix
 *  Initial revision
 *
 *  Revision 1.1  2002/07/26 00:25:00  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using Stochastix.Utilities;
using Stochastix.Numerics.Resources;

namespace Stochastix.Numerics.Distributions
{
	/// <summary>
	/// Non-central chi-square distribution.
	/// </summary>
	[ RcsId("$Id: NonCentralChiSquareDistribution.cs,v 1.1.1.1 2003/02/01 18:42:10 jens Exp $") ] 
	[ ComVisible(false) ]
	public class NonCentralChiSquareDistribution : IUnaryFunction
	{
		public NonCentralChiSquareDistribution(double df, double ncp) 
		{	
			this.df = df;
			this.ncp = ncp;
		}
		private double df, ncp;

		public double Value(double x)
		{
			if (x <= 0.0) return 0.0;

			const double errmax = 1e-12;
			const int    itrmax = 10000;
			double lam = 0.5*ncp;

			double u = Math.Exp(-lam);
			double v = u;
			double x2 = 0.5*x;
			double f2 = 0.5*df;
			double f_x_2n = df - x;

			double t = 0.0;
			if ( f2*Double.Epsilon > 0.125 &&
				Math.Abs(x2-f2) < Math.Sqrt(Double.Epsilon)*f2) 
			{
				t = Math.Exp( (1 - t) * (2 - t/(f2+1)) ) / 
					Math.Sqrt( 2.0 * Math.PI * (f2 + 1.0) );
			}
			else 
			{
				t = Math.Exp( f2 * Math.Log(x2) - x2 - 
					GammaFunction.LogValue(f2 + 1));
			}

			double ans = v*t;

			bool flag = false;
			int n = 1;
			double f_2n = df + 2.0;
			f_x_2n += 2.0;

			double bound;
			for (;;) 
			{
				if (f_x_2n > 0) 
				{
					flag = true;
					// oh my... f77.NET
					// goto L10;
					bound = t * x / f_x_2n;
					if (bound <= errmax || n > itrmax)
						goto L_End;
				}
				for (;;) 
				{
					u *= lam / n;
					v += u;
					t *= x / f_2n;
					ans += v*t;
					n++; 
					f_2n += 2.0;
					f_x_2n += 2.0;
					if (!flag && n <= itrmax)
						break;
				// L10:
					bound = t * x / f_x_2n;
					if (bound <= errmax || n > itrmax)
						goto L_End;
				}
			}
			L_End:
				if (bound > errmax)	// Didn't converge.
					throw new ApplicationException( Strings.Get(GetType(), "DistConv"));
			return (ans);

		}

	}

}
