// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: LinearInterpolation.cs,v $
 *  Revision 1.1.1.1  2003/02/01 18:42:10  jens
 *  Initial import
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Collections;
using Stochastix.Numerics.Resources;
using Stochastix.Utilities;

namespace Stochastix.Numerics.Interpolations
{
	/// <summary>
	/// Linear interpolation between discrete points.
	/// </summary>
	[ RcsId("$Id: LinearInterpolation.cs,v 1.1.1.1 2003/02/01 18:42:10 jens Exp $") ] 
	public class LinearInterpolation : Interpolation
	{
		/// <summary>
		/// Default constructor for this class.
		/// </summary>
		/// <overloads>
		/// Initialize a new linear interpolation.
		/// </overloads>
		/// <remarks>
		/// You need to <see cref="Initialize"/> a class constructed by the
		/// default constructor.
		/// </remarks>
		public LinearInterpolation() : base()
		{}

		/// <summary>
		/// Initialize a new linear interpolation.
		/// </summary>
		/// <remarks>
		/// The sequence of values for x must have been sorted.
		/// </remarks>
		/// <param name="x"></param>
		/// <param name="y"></param>
		/// <exception cref="ArgumentException">
		/// Thrown when the passed lists do not have the same size
		/// or do not provide enough points to interpolate.
		/// </exception>
		public LinearInterpolation(IList x, IList y) : base(x, y)
		{
		}

		/// <summary>
		/// Initialize a class constructed by the default constructor.
		/// </summary>
		/// <remarks>
		/// The sequence of values for x must have been sorted.
		/// </remarks>
		/// <exception cref="ArgumentException">
		/// Thrown when the passed lists do not have the same size
		/// or do not provide enough points to interpolate.
		/// </exception>
		public override void Initialize( IList x, IList y)
		{
			base.Initialize(x, y);
			this.y = y;
		}

		protected IList y = null;

		/// <summary>
		/// Interpolated value.
		/// </summary>
		/// <param name="x"></param>
		/// <param name="allowExtrapolation"></param>
		/// <returns>The interpolated value.</returns>
		/// <exception cref="ArgumentOutOfRangeException">
		/// Thrown when extrapolation has not been allowed and the passed value
		/// is outside the allowed range.
		/// </exception>
		/// <exception cref="InvalidOperationException">
		/// Thrown when the class was not properly initialized.
		/// </exception>
		override public double Value( double x, bool allowExtrapolation )
		{
			int position = Locate(x, allowExtrapolation);
			double yl = (double)this.y[position];
			double dy = (double)this.y[position+1] - yl;
			double xl = this.x[position];
			double dx = this.x[position+1] - xl;
			return yl + ( x-xl ) * dy / dx;
		}

	}
}
