// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: IObjectiveFunction.cs,v $
 *  Revision 1.1.1.1  2003/02/01 18:42:12  jens
 *  Initial import
 *
 *  Revision 1.4  2002/12/05 15:57:52  stochastix
 *  synced with my current sandbox
 *
 *  Revision 1.3  2002/10/15 12:44:45  stochastix
 *  updated license
 *
 *  Revision 1.2  2002/08/28 15:36:34  stochastix
 *  changes due to FxCop
 *
 *  Revision 1.1.1.1  2002/08/15 11:44:11  stochastix
 *  Initial revision
 *
 *  Revision 1.3  2002/07/24 17:04:24  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.2  2002/07/24 14:42:41  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using Stochastix.Utilities;

namespace Stochastix.Numerics.Solvers1D
{
	/// <summary>
	/// Interface to objective function for 1-D solvers.
	/// </summary>
	/// <remarks>
	/// This is the function whose zeroes must be found.
	/// </remarks>
	[ RcsId("$Id: IObjectiveFunction.cs,v 1.1.1.1 2003/02/01 18:42:12 jens Exp $") ] 
	[ ComVisible(true) ]
	[ Description("Interface to objective function for 1-D solvers.") ]
	[ InterfaceTypeAttribute(ComInterfaceType.InterfaceIsDual) ]
	public interface IObjectiveFunction
	{
		/// <summary>
		/// A stringified representation of the objective function.
		/// </summary>
		/// <returns>A String representing the object.</returns>
		[
		Description("A stringified representation of this object."),
		DispId(0)	// DISPID_VALUE
		]
		string ToString();

		/// <summary>
		/// Definiton of the objective function.
		/// </summary>
		/// <param name="x">Argument to the objective function.</param>
		/// <returns>The value of the objective function, <i>f(x)</i>.</returns>
		[ Description("Definiton of the objective function.") ]
		double Value(double x);

		/// <summary>
		/// Derivative of the objective function.
		/// </summary>
		/// <param name="x">Argument to the objective function.</param>
		/// <returns>The value of the derivative, <i>f'(x)</i>.</returns>
		/// <exception cref="NotImplementedException">
		/// Thrown when the function's derivative has not been implemented.
		/// </exception>
		[ Description("Derivative of the objective function.") ]
		double Derivative(double x);
	}
}



