// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: TestBLAS.cs,v $
 *  Revision 1.1.1.1  2003/02/01 18:42:13  jens
 *  Initial import
 *
 *  Revision 1.3  2002/12/05 15:57:51  stochastix
 *  synced with my current sandbox
 *
 *  Revision 1.2  2002/10/15 12:53:07  stochastix
 *  updated license
 *
 *  Revision 1.1  2002/08/28 15:43:35  stochastix
 *  moved into dotNum module
 *
 *  Revision 1.2  2002/08/20 22:33:28  stochastix
 *  Using more general TypeLoadException
 *
 *  Revision 1.1  2002/08/20 16:46:18  stochastix
 *  updated
 *
 *
 *********************************************************************
 */
#endregion

namespace Stochastix.Numerics.Tests
{
	using System;
	using System.Collections;
	using NUnit.Framework;
	using Org.Mentalis.Utilities;
	using Stochastix.Numerics;
	using Stochastix.Numerics.LA;

	/// <summary>
	/// 
	/// </summary>
	public class TestBLAS: TestCase 
	{
		/// <summary>
		/// Just a constructor. I think you can pass the name of the
		/// test (==method) to run.
		/// </summary>
		/// <param name="name">Name of the test to run.</param>
		public TestBLAS(String name): base(name) {}

		// declare private members here used for test setup

		double[] indexValues;
		double[] ones;

		private const double tolerance = 1e-8;
		
		/// <summary>
		/// Initialize this test case.
		/// </summary>
		protected override void SetUp() 
		{
			indexValues = new double[]
				{ 0.5, -100.0, 0.001, 1.0, -0.5, 0.001, 0.001, 0.001, 0.001 };
			ones = new double[100];
			for(int i=0; i<ones.Length; i++)
				ones[i] = 1.0;
		}

		/// <summary>
		/// Free all you resources.
		/// </summary>
		protected override void TearDown()
		{
		}

		public void TestIndices()
		{
			int n = indexValues.Length;
			int inc = 1;
			double[] clone = (double[])indexValues.Clone();

			Assertion.Assert("iamin", 2 ==
				BLAS.iamin(n, ref indexValues[0], inc));
//			Assertion.Assert("IDAMIN_unsafe", 3 ==
//				BLAS.IDAMIN_unsafe(ref n, ref indexValues[0], ref inc));

			Assertion.Assert("imin", 1 ==
				BLAS.imin(n, ref indexValues[0], inc));
//			Assertion.Assert("IDMIN_unsafe", 2 ==
//				BLAS.IDMIN_unsafe(ref n, ref indexValues[0], ref inc));

			Assertion.Assert("iamax", 1 ==
				BLAS.iamax(n, ref indexValues[0], inc));
//			Assertion.Assert("IDAMAX_unsafe", 2 ==
//				BLAS.IDAMAX_unsafe(ref n, ref indexValues[0], ref inc));

			Assertion.Assert("imax", 3 ==
				BLAS.imax(n, ref indexValues[0], inc));
//			Assertion.Assert("IDMAX_unsafe", 4 ==
//				BLAS.IDMAX_unsafe(ref n, ref indexValues[0], ref inc));

			try
			{
				Assertion.Assert("IDAMIN", 3 ==
					BLAS.IDAMIN(ref n, ref indexValues[0], ref inc));
				Assertion.Assert("IDMIN", 2 ==
					BLAS.IDMIN(ref n, ref indexValues[0], ref inc));
				Assertion.Assert("IDAMAX", 2 ==
					BLAS.IDAMAX(ref n, ref indexValues[0], ref inc));
				Assertion.Assert("IDMAX", 4 ==
					BLAS.IDMAX(ref n, ref indexValues[0], ref inc));
			} 
			catch ( System.TypeLoadException e) 
			{
				Console.WriteLine("Skipping native tests: {0}", e.Message);
			}

			Assertion.Assert( "pass-by-ref integrity of ints",
				n == indexValues.Length && inc == 1);
			for(int i=0; i<clone.Length; i++)
				Assertion.Assert( "pass-by-ref integrity of array",
					clone[i] == indexValues[i]);

		}

		public void TestSums()
		{
			int n = indexValues.Length;
			int inc = 1;
			double[] clone = (double[])indexValues.Clone();

			Assertion.Assert("sum", Math.Abs( -98.995 -
				BLAS.sum(n, ref indexValues[0], inc) ) <= tolerance);
//			Assertion.Assert("DSUM_unsafe", Math.Abs( -98.995 -
//				BLAS.DSUM_unsafe(ref n, ref indexValues[0], ref inc) ) <= tolerance);

			Assertion.Assert("asum", Math.Abs( 102.005 -
				BLAS.asum(n, ref indexValues[0], inc) ) <= tolerance);
//			Assertion.Assert("DASUM_unsafe", Math.Abs( 102.005 -
//				BLAS.DASUM_unsafe(ref n, ref indexValues[0], ref inc) ) <= tolerance);

			try
			{
				Assertion.Assert("DSUM", Math.Abs( -98.995 -
					BLAS.DSUM(ref n, ref indexValues[0], ref inc)) <= tolerance);
				Assertion.Assert("DASUM", Math.Abs( 102.005 -
					BLAS.DASUM(ref n, ref indexValues[0], ref inc) ) <= tolerance);
			} 
			catch ( System.TypeLoadException e) 
			{
				Console.WriteLine("Skipping native tests: {0}", e.Message);
			}

			Assertion.Assert( "pass-by-ref integrity of ints",
				n == indexValues.Length && inc == 1);
			for(int i=0; i<clone.Length; i++)
				Assertion.Assert( "pass-by-ref integrity of array",
					clone[i] == indexValues[i]);
		}


		public void TestDSCAL()
		{
			double[] temp;
			int n = ones.Length;
			int inc = 1;
			double zero = 0.0;
			double one = 1.0;
			double two = 2.0;

			temp = (double[])ones.Clone();
			BLAS.scal(n, zero, ref temp[0], inc);
			Assertion.Assert("scal 0.0", Math.Abs( zero*n -
				BLAS.sum(n, ref temp[0], inc) ) <= tolerance);

			temp = (double[])ones.Clone();
			BLAS.scal(n, one, ref temp[0], inc);
			Assertion.Assert("scal 1.0", Math.Abs( one*n -
				BLAS.sum(n, ref temp[0], inc) ) <= tolerance);
			
			temp = (double[])ones.Clone();
			BLAS.scal(n, two, ref temp[0], inc);
			Assertion.Assert("scal 2.0", Math.Abs( two*n -
				BLAS.sum(n, ref temp[0], inc) ) <= tolerance);

/* These are now private in BLAS
 
			temp = (double[])ones.Clone();
			BLAS.DSCAL_unsafe(ref n, ref zero, ref temp[0], ref inc);
			Assertion.Assert("scal 0.0", Math.Abs( zero*n -
				BLAS.sum(n, ref temp[0], inc) ) <= tolerance);

			temp = (double[])ones.Clone();
			BLAS.DSCAL_unsafe(ref n, ref one, ref temp[0], ref inc);
			Assertion.Assert("scal 1.0", Math.Abs( one*n -
				BLAS.sum(n, ref temp[0], inc) ) <= tolerance);
			
			temp = (double[])ones.Clone();
			BLAS.DSCAL_unsafe(ref n, ref two, ref temp[0], ref inc);
			Assertion.Assert("scal 2.0", Math.Abs( two*n -
				BLAS.sum(n, ref temp[0], inc) ) <= tolerance);

*/

			try
			{
				temp = (double[])ones.Clone();
				BLAS.DSCAL(ref n, ref zero, ref temp[0], ref inc);
				Assertion.Assert("scal 0.0", Math.Abs( zero*n -
					BLAS.sum(n, ref temp[0], inc) ) <= tolerance);

				temp = (double[])ones.Clone();
				BLAS.DSCAL(ref n, ref one, ref temp[0], ref inc);
				Assertion.Assert("scal 1.0", Math.Abs( one*n -
					BLAS.sum(n, ref temp[0], inc) ) <= tolerance);
			
				temp = (double[])ones.Clone();
				BLAS.DSCAL(ref n, ref two, ref temp[0], ref inc);
				Assertion.Assert("scal 2.0", Math.Abs( two*n -
					BLAS.sum(n, ref temp[0], inc) ) <= tolerance);
			} 
			catch ( System.TypeLoadException e) 
			{
				Console.WriteLine("Skipping native tests: {0}", e.Message);
			}

			Assertion.Assert( "pass-by-ref integrity of ints",
				n == ones.Length && inc == 1);
			Assertion.Assert( "pass-by-ref integrity of scalars",
				zero == 0.0 && one == 1.0 && two == 2.0);
		}

	}
}
