// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: TestMatrix.cs,v $
 *  Revision 1.1.1.1  2003/02/01 18:42:13  jens
 *  Initial import
 *
 *  Revision 1.3  2002/12/05 15:57:51  stochastix
 *  synced with my current sandbox
 *
 *  Revision 1.2  2002/10/15 12:53:08  stochastix
 *  updated license
 *
 *  Revision 1.1  2002/08/28 15:43:36  stochastix
 *  moved into dotNum module
 *
 *  Revision 1.2  2002/08/20 16:44:55  stochastix
 *  updated
 *
 *  Revision 1.1.1.1  2002/08/15 11:45:37  stochastix
 *  Initial revision
 *
 *  Revision 1.1  2002/08/07 19:15:21  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

namespace Stochastix.Numerics.Tests
{
	using System;
	using System.Collections;
	using NUnit.Framework;
	using Org.Mentalis.Utilities;
	using Stochastix.Numerics;


	/// <summary>
	/// 
	/// </summary>
	public class TestMatrix: TestCase 
	{
		/// <summary>
		/// Just a constructor. I think you can pass the name of the
		/// test (==method) to run.
		/// </summary>
		/// <param name="name">Name of the test to run.</param>
		public TestMatrix(String name): base(name) {}

		// declare private members here used for test setup

		private const double tolerance = 1e-8;
		
		private static Matrix AsymA = new Matrix( new double[,] 
		{	// Initiliazation from double[,]
			{ 0.658205507, 0.269658927, 0.531648039, 0.526713489, 0.195145154 },
			{ 0.447615044, 0.238673709, 0.225825010, 0.038748412, 0.731880500 },
			{ 0.759477576, 0.040596115, 0.905769217, 0.770803211, 0.009718652 }
		});
		
		private static Matrix AsymB = new Matrix(3, 5, new double[] 
		{	// Initialization from double[] array
			0.345321576,	0.697243111,	0.835915878,	0.741165240,	0.928399626,
			0.614524916,	0.460522543,	0.003223627,	0.870570040,	0.024052303,
			0.079690198,	0.848758725,	0.870769060,	0.034261879,	0.428992363
		});

		private static Matrix SymA = new Matrix( new double[,]
		{	// Initiliazation from double[,]
			{ 0.874158067,	0.169369246,	0.586688655,	0.080609188 },
			{ 0.569710384,	0.337007600,	0.962962235,	0.463437312 },
			{ 0.629171423,	0.303873197,	0.549995076,	0.712967454 },
			{ 0.264691808,	0.749569578,	0.734151748,	0.315645865 }
		});
		private static Matrix SymB = new Matrix( new double[,]
		{	// Initiliazation from double[,]
			{ 0.570654242,	0.280284655,	0.886921749,	0.669825815 },
			{ 0.317698955,	0.332761331,	0.260361239,	0.219054977 },
			{ 0.550256264,	0.278100560,	0.436929108,	0.292093911 },
			{ 0.865629816,	0.561748041,	0.138839909,	0.758076464 }
		});
		private static Matrix Unity = new Matrix( new double[,]
		{	// Initiliazation from double[,]
			{ 1.0,	0.0, 0.0, 0.0 },
			{ 0.0,	1.0, 0.0, 0.0 },
			{ 0.0,	0.0, 1.0, 0.0 },
			{ 0.0,	0.0, 0.0, 1.0 }
		});

		private static Vector VecX = new Vector( new double[]
		{
			0.447615044, 0.238673709, 0.22582501, 0.038748412
		});
		private static Vector VecY = new Vector( new double[]
		{
			0.195145154, 0.7318805, 0.009718652, 0.195145154
		});


		/// <summary>
		/// Initialize this test case.
		/// </summary>
		protected override void SetUp() 
		{
		}

		/// <summary>
		/// Free all you resources.
		/// </summary>
		protected override void TearDown()
		{
		}

		public void TestArrayCast()
		{
			Matrix mat = new Matrix(3,3);
			for( int i=0; i<3; i++)
				for( int j=0; j<3; j++)
					mat[i,j] = i*3 + j;

			double[,] matarray = (double[,])mat;
			for( int i=0; i<3; i++)
				for( int j=0; j<3; j++)
					Assertion.Assert(matarray[i,j] == i*3 + j);
		}

		public void TestArrayMultiplication()
		{
			// Note: I'm using the element sum as a kind of hash value here

			Assertion.Assert("Symmetric A * B", Math.Abs( 15.45955204 - 
				printMatrix( SymA * SymB )) < tolerance);
			Assertion.Assert("Symmetric A * B'", Math.Abs( 15.58519533 - 
				printMatrix( SymA * Matrix.Transpose(SymB) )) < tolerance);
			Assertion.Assert("Symmetric A' * B", Math.Abs( 14.97276293 - 
				printMatrix( Matrix.Transpose(SymA) * SymB )) < tolerance);

			Assertion.Assert("1 * scalar", Math.Abs( 6.0 - 
				printMatrix( Unity  * 1.5)) < tolerance);
			Assertion.Assert("Symmetric 1 * A", Math.Abs( 8.304008835 - 
				printMatrix( Unity * SymA )) < tolerance);
			Assertion.Assert("Symmetric A * 1", Math.Abs( 8.304008835 - 
				printMatrix( SymA * Unity )) < tolerance);

			Assertion.Assert("Symmetric A * y", Math.Abs( 1.932236207 - 
				printVector( SymA * VecY )) < tolerance);

			Assertion.Assert("Math expressions", Math.Abs( 
				(1.932236207 * 2.5 + printVector(VecX)) -
				printVector( SymA * VecY * 2.5 + VecX )) < tolerance);

			Assertion.Assert("Asymmetric A * B'", Math.Abs( 9.378025604 - 
				printMatrix( AsymA * Matrix.Transpose(AsymB) )) < tolerance);


		}

		public void TestAttachedVector()
		{
			// test the following properties of vectors
			//  standard collection interfaces
			//  mathematical operations
			// test the following properties of attached vectors
			//	vector has correct values
			//	vector can be modified like a usual vector
			//	modifying the vector does 
			//		change corresponding matrix values
			//		not change values in the remaining matrix
		/*
			Matrix e = Unity.Clone();
			Vector d = e.Diagonal;
			printVector( d );
			d.Multiply(1.5);
			printVector( d );
			printMatrix( e );
			d.Add(VecX);
			printMatrix( e );
			printVector(d.Clone().Add(VecY));
			printMatrix( e );
			for( int i=0; i< e.Rows; i++)
				printVector( e[i] );
		*/
		}


		private double printMatrix(Matrix m)
		{
			double sum = 0.0;
			foreach ( double d in (double[,])m)
				sum += d;
			// Console.WriteLine(m.ToString());
			return sum;
		}

		private double printVector(Vector v)
		{
			// TODO: Vector.ToString() (see Matrix.IFormatable)

			double sum = 0.0;
			// Console.Write("{ ");
			foreach (double d in v)
			{
				sum += d;
				// Console.Write("{0:f6} ", d);
			}
			// Console.WriteLine(" }");
			return sum;
		}


		public void DontTestBlasPerformance()
		{

			Vector test;
			// Vector result;
			Random rand = new Random();
			StopWatch s = new StopWatch();
			for( int d=1; d< 1000; d+=33 )
			{
				test = new Vector(d);
				// result = test;	// eliminate er
				for( int i=0; i<test.Count; i++)
						test[i] = rand.NextDouble() + 0.5;

				for( int k=0; k<10000; k++ )
				{
					if( k==3) s.Reset(); // three warmup rounds
					test.Multiply(1.0005);
					// result = test * 1.0005;
				}
				// have a check on the last run
				//for( int j=0; j<result.Count; j++)
				//	Assert( Math.Abs( result[j] - test[j] * 1.0005) <= 1e-10);
				Console.WriteLine("dim={0}, avg/100ms={1}", d, (double)s.Peek()/10.0);
			}
		}


	}
}
