// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: TestRiskStatistics.cs,v $
 *  Revision 1.1.1.1  2003/02/01 18:42:13  jens
 *  Initial import
 *
 *  Revision 1.3  2002/12/05 15:57:51  stochastix
 *  synced with my current sandbox
 *
 *  Revision 1.2  2002/10/15 12:53:47  stochastix
 *  added tests for random numbers and risk statistics
 *
 *
 *********************************************************************
 */
#endregion

namespace Stochastix.Numerics.Tests
{
	using System;
	using System.Collections;
	using NUnit.Framework;
	using Stochastix.Numerics.Statistics;

	/// <summary>
	/// 
	/// </summary>
	public class TestRiskStatistics: TestCase 
	{
		/// <summary>
		/// Just a constructor. I think you can pass the name of the
		/// test (==method) to run.
		/// </summary>
		/// <param name="name">Name of the test to run.</param>
		public TestRiskStatistics(String name): base(name) {}

		// declare private members here used for test setup

		/// <summary>
		/// Initialize this test case.
		/// </summary>
		protected override void SetUp() 
		{
			stat.AddRange(data);
		}

		private RiskStatistics stat = new RiskStatistics();

		private static double tol = 1e-9;

		private double[] data = 
			{
				0.0434315512407002, -0.2474082650833880,  0.4147556160444750,
				-0.4737678272134720,  0.3374959364872000, -0.2410807592120820,
				-0.4428092218556510,  0.3819591556474660,  0.4321946137654170,
				-0.2053712098384270,  0.2606751768079520,  0.3022384308926920,
				0.4949567971300990,  0.1820316147021910, -0.0561850097845635,
				0.4183459058015400, -0.2135321230760190,  0.1123462704131670,
				-0.4359512722844530, -0.2661181056125400, -0.3345882207505550,
				0.2204110087988120, -0.1321555868691760,  0.3113083578186830,
				-0.4292908627159880,  0.1457642662781690, -0.2378138820993390,
				-0.3588090976035450,  0.3471794355893340, -0.1394360771545360
			};

		/// <summary>
		/// Free all you resources.
		/// </summary>
		protected override void TearDown()
		{
			stat = null;
		}

		public void TestCount()
		{
			Assertion.Assert(stat.Count == data.Length);
		}

		public void TestWeightSum()
		{
			Assertion.Assert(
				Math.Abs(stat.WeightSum - (double)data.Length) < tol);
		}

		//////  The following Tests use Excel reference values  //////

		public void TestMin()
		{
			Assertion.Assert(stat.Min == -0.473767827213472);
		}

		public void TestMax()
		{
			Assertion.Assert(stat.Max == 0.494956797130099);
		}

		public void TestMean()
		{
			Assertion.Assert(
				Math.Abs( stat.Mean - 0.0063592205421387 ) < tol);
		}

		public void TestVariance()
		{
			Assertion.Assert(
				Math.Abs( stat.Variance - 0.101438857042941 ) < tol);
		}

		public void TestStandardDeviation()
		{
			Assertion.Assert(
				Math.Abs( stat.StandardDeviation - 0.31849467349226 ) < tol);
		}

		public void TestSkewness()
		{
			Assertion.Assert(
				Math.Abs( stat.Skewness - -0.000650903371128743 ) < tol);
		}

		public void TestKurtosis()
		{
			Assertion.Assert(
				Math.Abs( stat.Kurtosis - -1.533522043739540000 ) < tol);
		}

		// TODO: implement tests for the remaining mthods and properties


		/// <summary>
		/// Testing that correct exception is thrown from static methods.
		/// </summary>
		public void TestExceptionFromStaticMethod()
		{
			try
			{
				double result = RiskStatistics.ValueAtRisk(
					0.75, 0.0, 0.1);
			} 
			catch( Exception e) 
			{
				if ( e is ArgumentException) 
					return;
				else
					throw e;
			}
			Assertion.Fail("No exception thrown");
		}
	}
}
