// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: TestSegmentIntegral.cs,v $
 *  Revision 1.1.1.1  2003/02/01 18:42:13  jens
 *  Initial import
 *
 *  Revision 1.3  2002/12/05 15:57:51  stochastix
 *  synced with my current sandbox
 *
 *  Revision 1.2  2002/10/15 12:45:28  stochastix
 *  updated license
 *
 *  Revision 1.1  2002/08/28 15:43:36  stochastix
 *  moved into dotNum module
 *
 *  Revision 1.1.1.1  2002/08/15 11:45:37  stochastix
 *  Initial revision
 *
 *  Revision 1.1  2002/08/07 19:15:21  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

namespace Stochastix.Numerics.Tests
{
	using System;
	using System.Collections;
	using NUnit.Framework;
	using Stochastix.Numerics.Distributions;


	/// <summary>
	/// 
	/// </summary>
	public class TestSegmentIntegral: TestCase 
	{
		/// <summary>
		/// Just a constructor. I think you can pass the name of the
		/// test (==method) to run.
		/// </summary>
		/// <param name="name">Name of the test to run.</param>
		public TestSegmentIntegral(String name): base(name) {}

		// declare private members here used for test setup

		/// <summary>
		/// Initialize this test case.
		/// </summary>
		protected override void SetUp() 
		{
			testCases = new ArrayList();
			testCases.Add( new object[]	{	
				"f(x) = 1", 
				new UnaryFunction(one),
				0.0, 1.0, 1.0 
			});
			testCases.Add( new object[] {	
				"f(x) = x", 
				new UnaryFunction(identity),
				0.0, 1.0, 0.5 
			});
			testCases.Add( new object[] { 
				"f(x) = x^2", 
				new UnaryFunction(squared),
				0.0, 1.0, 1.0/3.0 
			});
			testCases.Add( new object[] {
				"f(x) = sin(x)", 
				new UnaryFunction(Math.Sin),
				0.0, Math.PI, 2.0 
			});
			testCases.Add( new object[] {
                "f(x) = cos(x)", 
				new UnaryFunction(Math.Cos),
				0.0, Math.PI, 0.0 
			});
			testCases.Add( new object[] {
                "f(x) = Gauss(x)", 
				new UnaryFunction(NormalDistribution.Function),
				-10.0, 10.0, 1.0 
			});
		}
		
		ArrayList testCases;
		const double tolerance = 1e-4;
		SegmentIntegral integrate = new SegmentIntegral(10000);

		/// <summary>
		/// Free all you resources.
		/// </summary>
		protected override void TearDown()
		{
			testCases = null;
		}

		private static double one(double x) { return 1.0; }
		private static double identity(double x) { return x; }
		private static double squared(double x) { return x*x; }

		public void TestIntegration()
		{
			foreach( object[] fn in testCases)
			{
				double numerical = integrate.Value( 
					(UnaryFunction)fn[1], 
					(double)fn[2], (double)fn[3]);
				Assert((string)fn[0], 
					Math.Abs( numerical- (double)fn[4]) <= tolerance);
			}
		}
	}
}
