// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: ShortFloatingCoupon.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:01:27  jens
 *  Initial import
 *
 *  Revision 1.4  2002/08/30 18:21:10  stochastix
 *  license and namespace tweaks
 *
 *  Revision 1.3  2002/07/24 17:04:26  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.2  2002/07/24 14:42:42  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using Stochastix.QuantLib.Indexes;
using Stochastix.Business;
using Stochastix.Business.Calendars;
using Stochastix.Utilities;

namespace Stochastix.QuantLib.CashFlows
{
	/// <summary>
	/// Short coupon at par on a term structure.
	/// </summary>
	[ RcsId("$Id: ShortFloatingCoupon.cs,v 1.1.1.1 2003/02/01 19:01:27 jens Exp $") ] 
	[ ComVisible(false) ]
	public class ShortFloatingCoupon : FloatingRateCoupon, IFloatingRateCoupon
	{

		/// <summary>
		/// Construct a short coupon. The coupon does not 
		/// roll the payment date which must already be a business day.
		/// </summary>
		/// <param name="nominal">Nominal</param>
		/// <param name="paymentDate">A pre-rolled business day.</param>
		/// <param name="index"></param>
		/// <param name="fixingDays"></param>
		/// <param name="spread"></param>
		/// <param name="startDate"></param>
		/// <param name="endDate"></param>
		public ShortFloatingCoupon(
			double nominal, DateTime paymentDate, 
			// CHANGED the order of params in the constructor
			IXibor index, int fixingDays, double spread,
			DateTime startDate, DateTime endDate)
			: this(nominal, paymentDate, index, fixingDays, spread,
				startDate, endDate,	
				CalendarBase.NullDate, CalendarBase.NullDate)
		{}
		
		/// <summary>
		/// Construct a short coupon. The coupon does not 
		/// roll the payment date which must already be a business day.
		/// </summary>
		/// <param name="nominal">Nominal</param>
		/// <param name="paymentDate">A pre-rolled business day.</param>
		/// <param name="index"></param>
		/// <param name="fixingDays"></param>
		/// <param name="spread"></param>
		/// <param name="startDate"></param>
		/// <param name="endDate"></param>
		/// <param name="refPeriodStart"></param>
		/// <param name="refPeriodEnd"></param>
		public ShortFloatingCoupon(
			double nominal, DateTime paymentDate,
			// CHANGED the order of params in the constructor
			IXibor index, int fixingDays, double spread,
			DateTime startDate, DateTime endDate,
			DateTime refPeriodStart, DateTime refPeriodEnd )
			: base(nominal, paymentDate, index, fixingDays, spread,
				startDate, endDate,	refPeriodStart, refPeriodEnd)
		{}

		override public double Amount
		{
			get
			{
				DateTime todaysDate = Index.TermStructure.TodaysDate;
				DateTime fixingDate = Index.Calendar.Advance(
					AccrualStartDate, -FixingDays, TimeUnit.Days,
					RollingConvention.Preceding); 

				if( fixingDate < todaysDate ) 
				{
					// must have been fixed
					// but we have no way to interpolate the fixing yet
					throw new ApplicationException( String.Format(
						"short/long floating coupons not supported yet (start={0}, end={1})",
						AccrualStartDate.ToShortDateString(),
						AccrualEndDate.ToShortDateString()
						)
					);
				}
				return base.Amount;
			}
		}
	}
}
