// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: SimpleCashFlow.cs,v $
 *  Revision 1.2  2003/02/04 11:52:42  jens
 *  made ctors public
 *
 *  Revision 1.1.1.1  2003/02/01 19:01:27  jens
 *  Initial import
 *
 *  Revision 1.4  2002/08/30 18:21:10  stochastix
 *  license and namespace tweaks
 *
 *  Revision 1.3  2002/07/24 17:04:26  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.2  2002/07/24 14:42:42  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using Stochastix.Utilities;

namespace Stochastix.QuantLib.CashFlows
{

	/// <summary>
	/// Simple cash flow, which pays pays a predetermined amount at 
	/// a given date.
	/// </summary>
	[ RcsId("$Id: SimpleCashFlow.cs,v 1.2 2003/02/04 11:52:42 jens Exp $") ] 
	[ ComVisible(true) ]
	[ Description("Simple cash flow, which pays pays a predetermined amount at a given date.") ]
	[ ClassInterface(ClassInterfaceType.None) ]
	public class SimpleCashFlow : Observable, ISimpleCashFlow
	{
		private double amount;
		private DateTime date;

		public SimpleCashFlow(double amount, DateTime date)
		{
			this.amount = amount;
			this.date = date;
		}

#region Implementation of ISimpleCashFlow

		override public string ToString()
		{
			return string.Format( "{0:d}: {1:n}", date,	amount);
		}

		public double Amount
		{
			get	{ return amount; }
			set
			{
				amount = value;
				OnChanged();
			}
		}

		public DateTime Date
		{
			get	{ return date; }
		}
#endregion
	}
}
