// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: BSMOperator.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:01:27  jens
 *  Initial import
 *
 *  Revision 1.2  2002/08/30 18:13:01  stochastix
 *  small changes for DotNum and license
 *
 *  Revision 1.1  2002/08/07 19:14:59  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using Stochastix.Utilities;

namespace Stochastix.QuantLib.FiniteDifferences
{
	/// <summary>
	/// Black-Scholes-Merton differential operator.
	/// </summary>
	[ RcsId("$Id: BSMOperator.cs,v 1.1.1.1 2003/02/01 19:01:27 jens Exp $") ] 
	[ ComVisible(false) ]
	[ Description("Black-Scholes-Merton differential operator.") ]
	public class BSMOperator : TridiagonalOperator, IOperator
	{
		// public BSMOperator() : base() {}
		public BSMOperator(int size, double dx, double r,
			double q, double sigma) : base(size) 
		{
			double sigma2 = sigma*sigma;
			double nu = r-q-sigma2/2;
			double pd = -(sigma2/dx-nu)/(2*dx);
			double pu = -(sigma2/dx+nu)/(2*dx);
			double pm = sigma2/(dx*dx)+r;
			SetMidRows(pd,pm,pu);
		}
	}
}
