// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: IXibor.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:01:29  jens
 *  Initial import
 *
 *  Revision 1.4  2002/08/30 18:21:10  stochastix
 *  license and namespace tweaks
 *
 *  Revision 1.3  2002/07/24 17:04:25  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.2  2002/07/24 14:42:42  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using Stochastix.Business;
using Stochastix.Utilities;

namespace Stochastix.QuantLib.Indexes
{
	/// <summary>
	/// Interface to the popular X-ibor index classes.
	/// </summary>
	/// <remarks>
	/// This interface is mainly for COM interop.
	/// </remarks>
	/// <seealso cref="IXiborHistory"/>
	[ RcsId("$Id: IXibor.cs,v 1.1.1.1 2003/02/01 19:01:29 jens Exp $") ] 
	[ ComVisible(true) ]
	[ Description("Interface to the popular X-ibor index classes.") ]
	[ InterfaceTypeAttribute(ComInterfaceType.InterfaceIsDual) ]
	public interface IXibor : IIndex
	{

		#region Repeated declaration of IIndex

		/// <summary>
		/// A stringified representation of the Index.
		/// </summary>
		/// <returns>A String representing the object.</returns>
		[ Description("A stringified representation of this object.") ]
		[ DispId(0) ]	// DISPID_VALUE
		new String ToString();

		/// <summary>
		/// Index fixing at the given date.
		/// </summary>
		/// <remarks>
		/// <para>
		/// Any date passed as arguments must be a value date,
		/// i.e., the real calendar date advanced by a number of
		/// settlement days.
		/// </para>
		/// </remarks>
		/// <param name="fixingDate">The fixing date as a value date.</param>
		/// <returns>The fixing at the given date.</returns>
		new double Fixing(DateTime fixingDate);

		#endregion

		/// <summary>
		/// Index family name.
		/// </summary>
		string FamilyName
		{
			get;
		}

		/// <summary>
		/// Index tenor as <see cref="Period"/>.
		/// </summary>
		Period Tenor
		{
			get;
		}

		/// <summary>
		/// Settlement days.
		/// </summary>
		int SettlementDays
		{ 
			get;
		}
        
		/// <summary>
		/// Index <see cref="Stochastix.Business.Currency"/>.
		/// </summary>
		Currency Currency
		{ 
			get;
		}
        
		/// <summary>
		///	<see cref="IBusinessCalendar"/> used for advancing dates and 
		///	rolling holidays to business days.
		/// </summary>
		IBusinessCalendar Calendar
		{ 
			get;
		}
        
		bool IsAdjusted
		{ 
			get;
		}
        
		/// <summary>
		///	Algorithm used to find the business day which is "closest" to a given holiday.
		///	See <see cref="Stochastix.Business.RollingConvention"/> for a list of possible values.
		/// </summary>
		RollingConvention RollingConvention
		{
			get;
		}
        
		/// <summary>
		///	<see cref="IDayCounter"/> used for internal date/time conversions.
		/// </summary>
		IDayCounter DayCounter
		{ 
			get;
		}
        
		/// <summary>
		/// <see cref="ITermStructure"/> used for forecasting index fixings.
		/// </summary>
		ITermStructure TermStructure
		{
			get;
		}

		/// <summary>
		/// Fixing frequency in 1/years.
		/// </summary>
		/// <value>The fixing frequency in times/year</value>
		/// <exception cref="InvalidOperationException">
		/// Thrown when the period is not one of 1m, 2m, 3m, 4m, 6m, 12m and 1y.
		/// </exception>
		int Frequency
		{
			get;
		}

	}
}
