// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: BinomialTree.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:01:30  jens
 *  Initial import
 *
 *  Revision 1.4  2002/08/30 18:21:11  stochastix
 *  license and namespace tweaks
 *
 *  Revision 1.3  2002/07/24 17:04:23  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.2  2002/07/24 14:42:41  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using Stochastix.Utilities;

namespace Stochastix.QuantLib.Lattices
{
	/// <summary>
	/// Binomial tree abstract base class.
	/// </summary>
	/// <remarks>
	/// <para>
	/// The binomial method is the simplest numerical method that can be used to
	/// price path-independent derivatives. It is usually the preferred lattice
	/// method under the Black-Scholes-Merton model. 
	/// See <see cref="BlackScholesLattice"/> for a class based on 
	/// a <see cref="BinomialTree"/>.
	/// </para>
	/// <para>
	/// There are several approaches to build the underlying binomial tree,
	/// like <see cref="JarrowRudd">Jarrow-Rudd</see> or 
	/// <see cref="CoxRossRubinstein">Cox-Ross-Rubinstein</see>.
	/// </para>
	/// </remarks>
	/// <seealso cref="StandardBinomialTree"/>
	/// <seealso cref="JarrowRudd"/>
	/// <seealso cref="CoxRossRubinstein"/>
	[ RcsId("$Id: BinomialTree.cs,v 1.1.1.1 2003/02/01 19:01:30 jens Exp $") ] 
	[ ComVisible(false) ]
	public abstract class BinomialTree : Tree 
	{
		public BinomialTree(int nColumns) : base(nColumns) 
		{}

		override public int Count(int i)
		{
			return i+1;
		}

		override public int Descendant(int i, int index, int branch)
		{
			return index + branch;
		}
	}
}
