// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: INumericalMethod.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:01:31  jens
 *  Initial import
 *
 *  Revision 1.4  2002/08/30 18:21:11  stochastix
 *  license and namespace tweaks
 *
 *  Revision 1.3  2002/07/24 17:04:27  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.2  2002/07/24 14:42:42  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using System.Collections;
using Stochastix.Utilities;

namespace Stochastix.QuantLib.NumericalMethods
{
	/// <summary>
	/// Numerical method (Tree, Finite Differences) interface.
	/// </summary>
	[ RcsId("$Id: INumericalMethod.cs,v 1.1.1.1 2003/02/01 19:01:31 jens Exp $") ] 
	[ ComVisible(true) ]
	[ Description("Numerical method (Tree, Finite Differences) interface.") ]
	[ InterfaceTypeAttribute(ComInterfaceType.InterfaceIsDual) ]
	public interface INumericalMethod
	{
		// TODO: Add ToString() method

		ITimeGrid TimeGrid
		{ 
			get; 
		}

		/// <summary>
		/// Initialize a DiscretizedAsset object.
		/// </summary>
		/// <param name="asset"></param>
		/// <param name="time"></param>
		void Initialize(IDiscretizedAsset asset, double time);

		/// <summary>
		/// Roll-back a DiscretizedAsset object until a certain time.
		/// </summary>
		/// <param name="asset"></param>
		/// <param name="toTime"></param>
		void Rollback(IDiscretizedAsset asset, double toTime);
	}
}
