// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: Constraint.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:01:31  jens
 *  Initial import
 *
 *  Revision 1.4  2002/10/15 13:20:40  stochastix
 *  now using dotNum
 *
 *  Revision 1.3  2002/08/30 18:21:11  stochastix
 *  license and namespace tweaks
 *
 *  Revision 1.2  2002/08/15 18:14:20  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.1  2002/08/07 19:14:59  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using Stochastix.Numerics;
using Stochastix.Utilities;
using Stochastix.QuantLib.Resources;


namespace Stochastix.QuantLib.Optimization
{
	/// <summary>
	/// Abstract constraint class.
	/// </summary>
	[ RcsId("$Id: Constraint.cs,v 1.1.1.1 2003/02/01 19:01:31 jens Exp $") ] 
	[ ComVisible(false) ]
	public abstract class Constraint
	{
		public Constraint() 
		{}

		/// <summary>
		/// Tests if <paramref name="parameters"/> satisfy the constraint.
		/// </summary>
		public abstract bool Test(Vector parameters);
	
		public double Update(Vector parameters, Vector direction, double beta)
		{
			for( int iter=0; iter<200; iter++)
			{
				// newParams = parameters + diff*direction;
				Vector newParams = parameters.Clone();
				newParams.Add(direction, beta);
				if( Test(newParams) ) 
				{
					parameters.Add(direction, beta);
					return beta;
				}
				beta *= 0.5;
			}
			throw new ApplicationException(
				Strings.Get( GetType(), "OptConstrUpd"));
		}
	}
}
