// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: AutoColumnAttribute.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:01:32  jens
 *  Initial import
 *
 *  Revision 1.2  2002/08/30 18:21:11  stochastix
 *  license and namespace tweaks
 *
 *  Revision 1.1  2002/08/07 19:14:59  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using System.Data;
using Stochastix.Utilities;

namespace Stochastix.QuantLib.PricingEngines
{
	/// <summary>
	/// Attribute to build <see cref="System.Data.DataColumn"/> 
	/// objects at runtime.
	/// </summary>
	[ AttributeUsage( AttributeTargets.Field, Inherited = true ) ]
	[ RcsId("$Id: AutoColumnAttribute.cs,v 1.1.1.1 2003/02/01 19:01:32 jens Exp $") ] 
	[ ComVisible(false) ]
	public class AutoColumnAttribute : Attribute
	{
		public AutoColumnAttribute(string columnName,
			Type dataType)
		{
			this.columnName = columnName;
			this.dataType = dataType;
		}

		#region private members

		private string columnName;
		private Type dataType;

		private object defaultValue = null;
		private double minValue = double.MinValue;
		private double maxValue = double.MaxValue;
		private bool allowDBNull = false;
		private MappingType columnMapping = MappingType.Element;

		#endregion

		public DataColumn NewColumn()
		{
			DataColumn dc = new DataColumn(columnName, dataType);
			if(this.defaultValue != null)
				dc.DefaultValue = this.defaultValue;
			dc.ColumnMapping = this.columnMapping;
			dc.AllowDBNull = this.allowDBNull;
			return dc;
		}

		#region Properties

		public string ColumnName
		{ 
			get { return columnName; }
		}
		public Type DataType
		{ 
			get { return dataType; }
		}
		public object DefaultValue
		{
			get { return defaultValue; }
			set { defaultValue = value; }
		}
		public double Min
		{
			get { return minValue; }
			set { minValue = value; }
		}
		public double Max
		{
			get { return maxValue; }
			set { maxValue = value; }
		}
		public bool AllowDBNull
		{
			get { return allowDBNull; }
			set { allowDBNull = value; }
		}
		public MappingType ColumnMapping
		{ 
			get { return columnMapping; }
			set { columnMapping = value; }
		}

		#endregion

		public void Validate(DataRow row) 
		{
			if( row.IsNull(columnName) && this.AllowDBNull ) return;
			if( minValue > double.MinValue || maxValue < double.MaxValue )
			{
				double d;
				object o = row[columnName];
				if( o is double )
					d = (double)o;
				else if (o is int ) 
					d = System.Convert.ToDouble((int)o);
				else throw new ApplicationException("Column '" +
						 columnName + "': value is not numeric.");
				if( d<minValue || d>maxValue )
					throw new ApplicationException( "Column '" +
						columnName + "': value is out of allowed range.");
			}
		}
	}
}
