// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: ITermStructureFactory.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:01:37  jens
 *  Initial import
 *
 *  Revision 1.3  2002/08/30 18:21:12  stochastix
 *  license and namespace tweaks
 *
 *  Revision 1.2  2002/08/07 19:15:00  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.1  2002/07/31 18:58:10  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Collections;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using Stochastix.Business;
using Stochastix.Utilities;


namespace Stochastix.QuantLib.TermStructures
{
	/// <summary>
	/// Factory to build term structures.
	/// </summary>
	/// <remarks>
	/// This interface is mainly for COM interop.
	/// </remarks>
	[ RcsId("$Id: ITermStructureFactory.cs,v 1.1.1.1 2003/02/01 19:01:37 jens Exp $") ] 
	[ ComVisible(true) ]
	[ Description("Factory to build term structures.") ]
	[ InterfaceTypeAttribute(ComInterfaceType.InterfaceIsDual) ]
	public interface ITermStructureFactory 
	{
		[ DispId(1) ]
		[ Description("Items in the internal collection.") ] 
		int Count();

		[ Description("Default BusinessCalendar for all operations.") ] 
		IBusinessCalendar Calendar 
		{
			get; set;
		}

		/// <summary>
		/// The <see cref="IDayCounter"/> used for internal date/time conversions.
		/// </summary>
		[ Description("Default day counter for all operations.") ] 
		IDayCounter DayCounter
		{
			get; set;
		}

		[ Description("Default rolling convention.") ] 
		RollingConvention RollingConvention
		{
			get; set;
		}

		[ Description("Default settlement days.") ] 
		int SettlementDays
		{
			get; set;
		}

		[ Description("Build a flat forward termstructure from rate") ] 
		ITermStructure BuildFlatForward(String name, 
			IDayCounter dayCounter, double forwardRate, 
			DateTime todaysDate);

		[ Description("Build a ") ]	// TODO
		ITermStructure BuildZeroSpreaded(String name, IDayCounter dayCounter, 
			ITermStructure originalCurve, double spread);

		[ Description("Build a ") ]	// TODO
		ITermStructure BuildForwardSpreaded(String name, IDayCounter dayCounter,
			ITermStructure originalCurve, double spread);

		[ Description("Reset internal factory collection.") ] 
		void Reset();

		[ Description("Adds a deposit rate.") ] 
		void AddDepositRate(String tenor, double rate, IDayCounter dayCounter, int settlementDays);
		
		[ Description("Build a piecewise flat forward termstructure from added rates") ]
		ITermStructure BuildPiecewiseFlatForward(String name, 
			IDayCounter dayCounter, DateTime todaysDate);

	}
}
