// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: TermStructureFactory.cs,v $
 *  Revision 1.2  2003/02/04 11:52:42  jens
 *  made ctors public
 *
 *  Revision 1.1.1.1  2003/02/01 19:01:38  jens
 *  Initial import
 *
 *  Revision 1.6  2002/08/30 18:21:12  stochastix
 *  license and namespace tweaks
 *
 *  Revision 1.5  2002/08/07 19:15:00  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.4  2002/07/31 18:49:39  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.3  2002/07/24 17:04:26  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.2  2002/07/24 14:42:42  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Collections;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using Stochastix.QuantLib.MarketElements;
using Stochastix.QuantLib.TermStructures;
using Stochastix.Business;
using Stochastix.Business.Calendars;
using Stochastix.Business.DayCounters;
using Stochastix.Utilities;

namespace Stochastix.QuantLib.COM
{
	/// <summary>
	/// Factory to build complex termstructures.
	/// </summary>
	[ RcsId("$Id: TermStructureFactory.cs,v 1.2 2003/02/04 11:52:42 jens Exp $") ] 
	[ ComVisible(true) ]
	[ Description("Factory to build term structures.") ]
	[ ClassInterface(ClassInterfaceType.None) ]
	public class TermStructureFactory : ITermStructureFactory
	{
		IBusinessCalendar	calendar;
		RollingConvention	rollingConvention;
		IDayCounter			dayCounter;
		int					settlementDays;
		ArrayList			ratesVector;

		public TermStructureFactory() : this(
			TARGET.Instance,
			RollingConvention.ModifiedFollowing,
			Actual365.Instance, 2)
		{}

		public TermStructureFactory(
			IBusinessCalendar calendar,
			RollingConvention rollingConvention,
			IDayCounter dayCounter,
			int settlementDays )
		{
			this.calendar = calendar;
			this.rollingConvention = rollingConvention;
			this.dayCounter = dayCounter;
			this.settlementDays = settlementDays;
			this.ratesVector = new ArrayList(15); // initial capacity
		}


		#region Implementation of ITermStructureFactory

		public ITermStructure BuildFlatForward(string name, 
			IDayCounter dayCounter, double forwardRate,
			DateTime todaysDate)
		{
			if( todaysDate == CalendarBase.NullDate)
				todaysDate = DateTime.Today;
			return new FlatForward( todaysDate, 
				calendar.Advance( todaysDate, settlementDays, 
					TimeUnit.Days,rollingConvention), 
				new SimpleMarketElement(forwardRate),
				dayCounter);
		}
		public ITermStructure BuildZeroSpreaded(string name, IDayCounter dayCounter, QuantLib.ITermStructure originalCurve, double spread)
		{
			return new ZeroSpreadedTermStructure(
				originalCurve, new SimpleMarketElement(spread)
			);
		}
		public ITermStructure BuildForwardSpreaded(string name, IDayCounter dayCounter, QuantLib.ITermStructure originalCurve, double spread)
		{
			return null;
	/* TODO:
			return new ForwardSpreadedTermStructure(
				originalCurve, new SimpleMarketElement(spread)
				);
	*/
				
		}
		public int Count()
		{
			return ratesVector.Count;
		}
		public void Reset()
		{
			ratesVector.Clear();
		}
		public void AddDepositRate(string tenor, double rate, 
			IDayCounter dayCounter, int settlementDays)
		{
			Period p = Period.Parse(tenor);
			ratesVector.Add( new DepositRateHelper( 
				new SimpleMarketElement(rate), 
				p.Length, p.Units, settlementDays, calendar, 
				rollingConvention, dayCounter));
		}
		public ITermStructure BuildPiecewiseFlatForward(
			string name, IDayCounter dayCounter,
			DateTime todaysDate)
		{
			if( todaysDate == CalendarBase.NullDate)
				todaysDate = DateTime.Today;
			return new PiecewiseFlatForward( todaysDate,  
				calendar.Advance( todaysDate, settlementDays, 
					TimeUnit.Days,rollingConvention),
				ratesVector, dayCounter, 1.0e-12 );
		}
		public IBusinessCalendar Calendar
		{
			get { return calendar; }
			set	{ calendar = value; }
		}

		public IDayCounter DayCounter
		{
			get	{ return dayCounter; }
			set	{ dayCounter = value; }
		}

		public RollingConvention RollingConvention
		{
			get { return rollingConvention; }
			set	{ rollingConvention = value; }
		}

		public int SettlementDays
		{
			get	{ return settlementDays; }
			set { settlementDays = value; }
		}
#endregion
	}
}
