// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: BlackVolatilityTermStructure.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:01:39  jens
 *  Initial import
 *
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using Stochastix.Business;
using Stochastix.Utilities;
using Stochastix.QuantLib.Resources;

namespace Stochastix.QuantLib.Volatilities
{

	/// <summary>
	/// Black volatility term structure abstract base class.
	/// </summary>
	/// <remarks>
	/// This abstract class acts as an adapter to BlackVolTermStructure
	/// allowing the programmer to implement only the
	/// <c>Volatility(double, double, bool)</c> method in derived classes.
	/// <para>Volatilities are assumed to be expressed on an annual basis.</para>
	/// </remarks>
	[ RcsId("$Id: BlackVolatilityTermStructure.cs,v 1.1.1.1 2003/02/01 19:01:39 jens Exp $") ] 
	[ ComVisible(false) ]
	public abstract class BlackVolatilityTermStructure : BlackVolTermStructure
	{
		/// <summary>
		/// Protected constructor for abstract base class.
		/// </summary>
		protected BlackVolatilityTermStructure()
		{}

		/// <summary>
		/// Black present (a.k.a spot) variance at a given time from reference.
		/// </summary>
		/// <remarks>
		/// Returns the variance for the given strike and date calculating it
		/// from the volatility.
		/// </remarks>
		/// <overloads>
		/// Black present (a.k.a spot) variance.
		/// </overloads>
		/// <param name="maturity">Time from reference.</param>
		/// <param name="strike"></param>
		/// <param name="extrapolate"></param>
		/// <returns>Variance at the given time.</returns>
		public override double Variance( double maturity, double strike, bool extrapolate)
		{
			double vol = Volatility( maturity, strike, extrapolate );
			return vol*vol*maturity;
		}

	}
}
