// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: ILocalVolTermStructure.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:01:39  jens
 *  Initial import
 *
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using Stochastix.Business;
using Stochastix.Utilities;

namespace Stochastix.QuantLib.Volatilities
{

	/// <summary>
	/// Local Volatility term structure interface.
	/// </summary>
	/// <remarks>
	/// Interface for concrete local volatility term structures.
	/// <para>Volatilities are assumed to be expressed on an annual basis.</para>
	/// </remarks>
	[ RcsId("$Id: ILocalVolTermStructure.cs,v 1.1.1.1 2003/02/01 19:01:39 jens Exp $") ] 
	[ ComVisible(true) ]
	[ Description("Interface for concrete local volatility term structures. " +
		  "Volatilities are assumed to be expressed on an annual basis.") ]
	[ InterfaceTypeAttribute(ComInterfaceType.InterfaceIsDual) ]
	public interface ILocalVolTermStructure : IObservable 
	{

		/// <summary>
		/// A stringified representation of this object.
		/// </summary>
		/// <returns>A String representing the object.</returns>
		[
		Description("A stringified representation of this object."),
		DispId(0)	// DISPID_VALUE
		]
		string ToString();

		/// <summary>
		/// Local volatility at a given date.
		/// </summary>
		/// <overloads>
		/// Local volatility.
		/// </overloads>
		/// <param name="date">The <see cref="DateTime"/> to use for calculations.</param>
		/// <param name="underlyingLevel"></param>
		/// <param name="extrapolate"></param>
		/// <returns>The local volatility at the given date</returns>
		[ Description("Local volatility at a given date.") ]
		double Volatility( DateTime date, double underlyingLevel, 
			bool extrapolate);

		/// <summary>
		/// Local volatility at a given time from reference.
		/// </summary>
		/// <param name="time">Time from reference.</param>
		/// <param name="underlyingLevel"></param>
		/// <param name="extrapolate"></param>
		/// <returns>The local volatility at the given time.</returns>
		[ ComVisible(false) ]
		[ Description("Local volatility at a given time from reference.") ]
		double Volatility(double time, double underlyingLevel, bool extrapolate);

		/// <summary>
		/// The latest time for which the term structure can return vols.
		/// </summary>
		double MaxTime 
		{
			get;
		}
		
		/// <summary>
		/// The latest date for which the term structure can return vols.
		/// </summary>
		/// <value><see cref="DateTime"/></value>
		DateTime MaxDate 
		{
			get;
		}

		/// <summary>
		/// The reference date of this curve for which t=0.
		/// </summary>
		/// <value><see cref="DateTime"/></value>
		DateTime ReferenceDate 
		{
			get;
		}

		/// <summary>
		/// The <see cref="IDayCounter"/> used for internal date/time conversions.
		/// </summary>
		IDayCounter DayCounter 
		{
			get;
		}


	}

}