// This document is part of the DotNum class library.
// For more information, see http://www.jens-thiel.de/dotnet/dotnum
// or contact Jens Thiel <mailto:jens@stochastix.de>.

#region Copyright  2002 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001, 2002 Jens Thiel (http://www.jens-thiel.de/)
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions 
 * are met:
 *
 *   Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in
 *   the documentation and/or other materials provided with the 
 *   distribution.
 * 
 *   Neither the names of the copyright holders nor the names of the
 *   QuantLib Group and its contributors may be used to endorse or
 *   promote products derived from this software without specific
 *   prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: ran3.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:02:14  jens
 *  Initial import
 *
 *  Revision 1.1  2002/10/15 12:51:08  stochastix
 *  Initial implementation of a PRNG framework
 *  with support for Intel MKL/VSL.
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes

namespace Stochastix.Random.Generators
{
	/// <summary>
	/// Random number generator that inherits from the System.Random
	/// class. 
	/// </summary>
	/// <remarks>
	/// At least in Rotor, System.Random implements ran3 from
	/// Numerical Recipes in C, Chapter 7.1 which is attributed
	/// to Knuth.
	/// </remarks>
	[ RcsId("$Id: ran3.cs,v 1.1.1.1 2003/02/01 19:02:14 jens Exp $") ] 
	[ ComVisible(false) ]
	public abstract class ran3 : System.Random, IBasicRng
	{
		/// <summary>
		/// Constructor. A random seed will be chosen based on 
		/// the system clock.
		/// </summary>
		public ran3() : base()
		{}

		/// <summary>
		/// Constructor initializes the RNG with the given seed.
		/// </summary>
		/// <param name="seed"></param>
		public ran3(int seed) : base(seed)
		{}

		#region Implementation of ICloneable
		public object Clone()
		{
			return this.MemberwiseClone();
		}
		#endregion

		#region Implementation of IDisposable
		public void Dispose()
		{}
		#endregion

		public virtual void Next(double[] r, int start, int count)
		{
			for( count += start; start<count; start++)
				r[start] = Sample();
		}
		public void Next(double[] r, int start)
		{
			Next( r, start, r.GetUpperBound(0)-start+1);
		}
		public void Next(double[] r)
		{
			Next(r, r.GetLowerBound(0), r.Length);
		}
	}
}
