// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: IContinousRng.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:02:13  jens
 *  Initial import
 *
 *  Revision 1.2  2002/12/05 15:57:51  stochastix
 *  synced with my current sandbox
 *
 *  Revision 1.1  2002/10/15 12:51:08  stochastix
 *  Initial implementation of a PRNG framework
 *  with support for Intel MKL/VSL.
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using Stochastix.Utilities;

namespace Stochastix.Random
{
	/// <summary>
	/// Interface to a double-precision floating point random number generator.
	/// </summary>
	[ RcsId("$Id: IContinousRng.cs,v 1.1.1.1 2003/02/01 19:02:13 jens Exp $") ] 
	[ ComVisible(false) ]
	public interface IContinousRng
	{
		/// <summary>
		/// Returns a random number between 0.0 and 1.0.
		/// </summary>
		/// <returns>
		/// A double-precision floating point number greater than or equal to 0.0, 
		/// and less than 1.0.
		/// </returns>
		double NextDouble();

		/// <summary>
		/// Returns an <see cref="Array"/> of random number between 0.0 and 1.0.
		/// </summary>
		/// <param name="r">
		/// A double-precision floating point <see cref="Array"/> to be filled
		/// with random numbers number greater than or equal to 0.0, and less than 1.0.
		/// </param>
		void Next(double[] r);

		/// <summary>
		/// Returns an <see cref="Array"/> of random number between 0.0 and 1.0.
		/// </summary>
		/// <param name="r">
		/// A double-precision floating point <see cref="Array"/> to be filled
		/// with random numbers number greater than or equal to 0.0, and less than 1.0.
		/// </param>
		/// <param name="start">Index of the first elemnt to fill.</param>
		void Next(double[] r, int start);

		/// <summary>
		/// Returns an <see cref="Array"/> of random number between 0.0 and 1.0.
		/// </summary>
		/// <param name="r">
		/// A double-precision floating point <see cref="Array"/> to be filled
		/// with random numbers number greater than or equal to 0.0, and less than 1.0.
		/// </param>
		/// <param name="start">Index of the first elemnt to fill.</param>
		/// <param name="length">Number of random numbers to generate.</param>
		void Next(double[] r, int start, int length);
	}
}
